#include <QtGui>
#include <iostream>
#include "bsstructureview.h"
#include "math.h"

using namespace std;


BSStructureView::BSStructureView (QImage *im, BSDetector *sd)
{
  graylevelImage = im;
  currentImage = im;
  det = sd;
  int w = im->width ();
  int h = im->height ();
  imageInBack = true;
  gradDir = 0;
  gradImage = NULL;
  gradInBack = false;
  setBackgroundBrush (QBrush (*currentImage));
  setScene (new QGraphicsScene (0, 0, w, h));
  grid = new BSStructureItem (w, h, im, sd);
  scene()->addItem (grid);
  setWindowTitle (grid->itemTitle () + backgroundSource ());
}


BSStructureView::~BSStructureView ()
{
  scene()->removeItem (grid);
  delete grid;
}


void BSStructureView::setGradientImage (QImage *gim)
{
  gradImage = gim;
}


void BSStructureView::updateBackground (bool zoomChanged)
{
  if (imageInBack)
  {
    int w = currentImage->width ();
    int h = currentImage->height ();
    int zf = grid->zoomFactor ();
    if (zoomChanged) image = currentImage->scaled (w * zf, h * zf);
    QBrush qb (image);
    qb.setTransform (QTransform::fromTranslate (
          w / 2 - zf * (w / 2 + grid->focusX ()),
          h / 2 - zf * (h / 2 + grid->focusY ())));
    setBackgroundBrush (qb);
  }
}


void BSStructureView::toggleBackground ()
{
  imageInBack = ! imageInBack;
  if (imageInBack) setBackgroundBrush (QBrush (image));
  else setBackgroundBrush (QBrush (Qt::white));
}


void BSStructureView::toggleGradient ()
{
  if (currentImage == gradImage) currentImage = graylevelImage;
  else currentImage = gradImage;
  setWindowTitle (grid->itemTitle () + backgroundSource ());
}


void BSStructureView::toggleGradientType ()
{
  if (++gradDir == 3) gradDir = 0;
  if (currentImage == gradImage) currentImage = gradImage;
  setWindowTitle (grid->itemTitle () + backgroundSource ());
}


QString BSStructureView::backgroundSource () const
{
    if (currentImage != gradImage) return ("");
    else if (gradDir == 2) return (" (on gradient Y)");
    else if (gradDir == 1) return (" (on gradient X)");
    return (" (on gradient Magn)");
}


void BSStructureView::paint (QPainter *painter,
                             const QStyleOptionGraphicsItem *option,
                             QWidget *widget)
{
  Q_UNUSED (painter);
  Q_UNUSED (option);
  Q_UNUSED (widget);
}


bool BSStructureView::processKeyEvent (QKeyEvent *event)
{
  bool processed = false;
  switch (event->key ())
  {
    case Qt::Key_I : // Info
      if (event->modifiers () & Qt::ControlModifier) grid->switchScanDisplay ();
      else grid->toggleDisplay ((event->modifiers () & Qt::ShiftModifier) == 0);
      setWindowTitle (grid->itemTitle () + backgroundSource ());
      scene()->update ();
      update ();
      processed = false;
      break;

    case Qt::Key_V : // Info display
      grid->switchInfoDisplay ();
      scene()->update ();
      update ();
      processed = false;
      break;

    case Qt::Key_B : // Background
      if (event->modifiers () & Qt::ControlModifier) toggleGradientType ();
      else if (event->modifiers () & Qt::ShiftModifier) toggleGradient ();
      else toggleBackground ();
      updateBackground (true);
      scene()->update ();
      update ();
      processed = false;
      break;

    case Qt::Key_Plus : // Zoom in
      grid->zoomIn ();
      updateBackground (true);
      scene()->update ();
      update ();
      processed = false;
      break;

    case Qt::Key_Minus : // Zoom out
      grid->zoomOut ();
      updateBackground (true);
      scene()->update ();
      update ();
      processed = false;
      break;

    case Qt::Key_Left : // Go leftwards
      grid->shift (-1, 0);
      updateBackground (false);
      scene()->update ();
      update ();
      processed = false;
      break;

    case Qt::Key_Right : // Go rightwards
      grid->shift (1, 0);
      updateBackground (false);
      scene()->update ();
      update ();
      processed = false;
      break;

    case Qt::Key_Up : // Go upwards
      grid->shift (0, -1);
      updateBackground (false);
      scene()->update ();
      update ();
      processed = false;
      break;

    case Qt::Key_Down : // Go downwards
      grid->shift (0, 1);
      updateBackground (false);
      scene()->update ();
      update ();
      processed = false;
      break;

    case Qt::Key_C : // Increments the connected components min size
      det->incConnectedComponentMinSize (
            (event->modifiers () & Qt::ShiftModifier) == 0);
      update ();
      processed = true;
      break;

    case Qt::Key_G : // Increments the connected components min size
      det->switchConnectivityConstraint ();
      update ();
      processed = true;
      break;

    case Qt::Key_S : // Switch dynamic scans
      det->switchDynamicScans ();
      update ();
      processed = true;
      break;

    case Qt::Key_O : // Switch orthographic scans
      det->switchOrthoScans ();
      update ();
      processed = true;
      break;

    case Qt::Key_P : // Switch pre-filtering
      det->switchFiltering (0);
      update ();
      processed = true;
      break;

    case Qt::Key_F : // Switch final filtering
      det->switchFiltering (1);
      update ();
      processed = true;
      break;

    case Qt::Key_D : // Initial detection extension limitation
      det->switchInitialBounding ();
      update ();
      processed = true;
      break;

    case Qt::Key_W : // Fine tracks max width
      det->setFineTracksMaxWidth (det->fineTracksMaxWidth () +
        (event->modifiers () & Qt::ShiftModifier ? -1 : 1));
      update ();
      processed = true;
      break;

    case Qt::Key_K : // Output blurred segment min size
      det->setBSminSize (det->getBSminSize () +
        (event->modifiers () & Qt::ShiftModifier ? -1 : 1));
      update ();
      processed = true;
      break;

    case Qt::Key_H : // Pixel lack tolerence
      det->setPixelLackTolerence (det->getPixelLackTolerence () +
        (event->modifiers () & Qt::ShiftModifier ? -1 : 1));
      update ();
      processed = true;
      break;
  }
  return processed;
}
